// game.as

if (Object.Swim == null) {
	Object.Swim = {};
}

if (Object.Swim.Game == null) {
	Object.Swim.Game = {};
}

var GameObj = Object.Swim.Game;

GameObj.initGame = function (playerCount) {
	var showMe = true;
	
	findProto (Object.Swim.Game).implement (Object.Screen.Generic);
	this.initScreenGeneric(showMe);

	playerCount != null && Number(playerCount) >= 2 ?
		this.playerCount = Number (playerCount) :
		this.playerCount = 2;
		
	this.suits = [
		{  desc : "Hearts", 		value : 1 }
		,{ desc : "Diamonds", 	value : 2 }
		,{ desc : "Spades", 		value	: 3 }
		,{ desc : "Clovers", 	value :	4 }
	];
	
	this.values = [
		{  face : "Seven", 	value : 7  }
		,{ face : "Eight", 	value : 8  }
		,{ face : "Nine", 	value : 9  }
		,{ face : "Ten", 		value : 10 }
		,{ face : "Jack", 	value : 10 }
		,{ face : "Queen", 	value : 10 }
		,{ face : "King", 	value : 10 }
		,{ face : "Ace", 		value : 11 }
	];
	
	this.cards 					= [];
	this.cardsPerHand 	= 3;
	
	this.maxChances 		= 3;
	
	this.winningScore		= 31;
	
	this.gameLog 				= [];
	
	var card, suitDesc, suitValue, face, faceValue;
	var id = 0;
	
	// create cards
	for (var s=0; s<this.suits.length; s++) {
		suitDesc = this.suits[s].desc;
		suitValue = this.suits[s].value;
		
		// trace (s + ", " + suit + ", " + suitValue);
		
		for (var v=0; v<this.values.length; v++) {
			face 			= this.values[v].face;
			faceValue = this.values[v].value;
			
			// trace (c + ", " + face + ", " + faceValue);
			
			id++;
			
			this.attachMovie ("card", suitDesc + face, id);
			card = this[suitDesc + face];
			card.implement (Object.Swim.Card);
			card.initCard (id, suitValue, faceValue, suitDesc , face);
			this.cards.push (card);
		}
	}
	
	// initialize players
	this.players = [];
	var myPlayer;
	
	for (i = 0; i<this.playerCount; i++) {
		myPlayer = this["player" + i];
		myPlayer.implement (Object.Swim.HumanPlayer);
		myPlayer.initHumanPlayer (i, this.cardsPerHand, this.maxChances);
		
		this.players.push (myPlayer);
	}
	
	// initialize stack
	
	this.stack.implement (Object.Swim.Hand);
	this.stack.initHand (this.cardsPerHand);
	
	// initialize screens
	
	this.screens.implement (Object.Swim.GameScreens);
	this.screens.initGameScreens(true, "title"); 

} // initGame

#include "includes/deck.as"

GameObj.getNextPlayer = function (currentPlayer) {
	var nextPlayer = currentPlayer;
	var found = false;
	
	while (!found) {
		nextPlayer== null || nextPlayer >= this.playerCount-1 ?
				nextPlayer = 0 :
				nextPlayer++;
			
		if (this.players[nextPlayer].getChances() >= 0) {
			found = true;
		}
	}	
	return nextPlayer;
} // nextPlayer

GameObj.deal = function () {
	this.deck.initDeck (this.cards);
	var card, player, myPlayer, i;
	
	for (i = 0; i<this.cardsPerHand; i++) {
		for (player = 0; player<this.playerCount; player++) {
			myPlayer = this.players[player];
			if (myPlayer.getChances() >= 0) {
				card = this.deck.drawCard();		
				myPlayer.setCard (i, card);
			}
		}
	}
	
	for (player = 0; player<this.playerCount; player++) {	
		myPlayer = this.players[player];
		if (myPlayer.getChances() >= 0) {
			myPlayer.analyzeHand();
		}
	}
	
	var stackCards = this.deck.drawCards (this.cardsPerHand);
	
	this.stack.setAllCards (stackCards);
	// this.stack.analyzeHand();
	
	if (this.dealer == null) {
		this.dealer = this.playerCount - 1;
	}
	else {
		this.dealer = this.getNextPlayer (this.dealer);
	}
} // deal

GameObj.resetGame = function () {
	var i;
	
	this.gameLog 				= [];
	
	this.dealer = null;
	this.rounds = 0;
	this.donePlayers = 0;
	
	// reset players
	for (i=0; i<this.playerCount; i++) {
		this.players[i].reset();
	}
	
	this.stack.hide();
	
	// remove all cards
	var deckLength = this.cards.length;
	for (i=0; i<deckLength; i++) {
		this.cards[i].hide();
	}
} // resetGame

GameObj.startGame = function () {
	// trace (_name + " startGame");

	var i;
	
	this.currentTurn 	= null;
	this.passCount 		= 0;
	this.knockPlayer 	= null;
	
	this.rounds == null ? 
		this.rounds = 1 :
		this.rounds++;
	
	this.gameLog 			= [];
	
	var myPlayer;
	
	for (var i=0; i<this.playerCount; i++) {
		myPlayer = this.players[i];
		if (myPlayer.getChances() >= 0) {
			myPlayer.startRound();
		}
	}
	
	this.deal();
	this.firstTurn();
} // startGame

GameObj.hideGame = function () {
	this.players[this.currentTurn].hide();
	this.stack.hide();
} // hideGame

GameObj.showGame = function () {
	if (this.currentTurn != null) {
		this.players[this.currentTurn].show();
		this.stack.show();
	}
} // showGame

GameObj.getNewCards = function () {
	var oldCards = this.stack.getAllCards();
	this.deck.discardCards (oldCards);
	
	var newCards = this.deck.drawCards (this.cardsPerHand);
	this.stack.setAllCards(newCards);
	
	return newCards;
} // getNewCards

GameObj.getKnockPlayer = function () {
	return this.knockPlayer;
} // getKnockPlayer

GameObj.getCardsDesc = function (cardsArray) {
	var desc = "";
	var card;
	
	for (var i=0; i<this.cardsPerHand; i++) {
		card = cardsArray[i];
		desc += card.getDesc();
		if (i<this.cardsPerHand-1) {
			desc += ", ";
		}
	}
	return desc;
} // getCardsDesc

GameObj.trackGame = function (myType, arg1, arg2, arg3, arg4) {
	var trackObj = { type : myType };
	
	if (myType == "move") {
		trackObj.player 	= this.players[arg1];
		trackObj.move 		= arg2;
		
		if (arg2 == "takeOne") {
			trackObj.slot = arg3;
			trackObj.card = arg4;
			
		} 
		else  if (arg2 == "takeAll") {
			trackObj.cards = arg3;
		}
		/*
		else if (arg2 == "pass") {
		} 
		else if (arg2 == "knock") {
		}*/
	}
	else if (myType == "newCards") {
		trackObj.cards = arg1;
	}
	else if (myType == "message") {
		trackObj.message = arg1;
	}
	
	this.gameLog.unshift (trackObj);
} // trackGame

GameObj.recordMove = function (move, arg1, arg2) {
	
	move == "pass" || move == "knock" ? 
		this.passCount++ :
		this.passCount = 0;
	
	if (move == "knock") {
		if (this.knockPlayer == null) {
			this.knockPlayer = this.currentTurn;
		}
	} 

	this.trackGame ("move", this.currentTurn, move, arg1, arg2);
} // recordMove

GameObj.finishGame = function (msg, loser) {
	var track;
	
	msg != null ? 
		track = msg + newline : 
		track = ""; 
		
	var score, lowestScore;
	var i;
	var losers = [];
	
	// find loser if not set already
	if (loser == null) {
		for (i=0; i<this.playerCount; i++) {
			score = this.players[i].getScore ();
			
			if ( score < this.winningScore && 
						( lowestScore == null || score < lowestScore ) ) {
				// found a new lowest score
				lowestScore = score;
				losers = [];
				losers[0] = i;
			} 
			else if (score == lowestScore) {
				// found another lowest score
				losers.push (i);
			}
		}	
	} 
	else {
		losers[0] = loser;
	}
	
	var temp;
  var myPlayer, chances;
	
	for (i=0; i<losers.length; i++) {
		myPlayer = this.players[ losers[i] ];
		chances = myPlayer.loseChance();
		
		track += myPlayer.getPlayerName() + " (" +
							getCardsDesc ( myPlayer.getAllCards() )  +") loses and ";
		
		if (chances > 1) {
			track += "has "+chances+" chances left";
		} else if (chances == 1) {
			track += "has "+chances+" chance left";
		} else if (chances == 0) {
			track += "is swimming";
		} else {
			track += "has drowned";
			this.donePlayers++;
		}
		track += "." + newline;
	}
	
	if (this.donePlayers < this.playerCount - 1) {
		// two or more players are still in the game
		this.screens.gotoPage ("endRound", track, this.rounds);
	} else {
		// game over - only one survivor
		var myPlayer;
		for (i=0; i<this.playerCount; i++) {
			myPlayer = this.players[i];
			if (myPlayer.getChances () >= 0) {
				track = myPlayer.getPlayerName() + " wins the series!" + newline;
				track += "(" + getCardsDesc ( myPlayer.getAllCards() ) + ")";
				this.screens.gotoPage ("endGame", track, this.rounds);
				break;
			}
		}	
	}
	
	this.currentTurn = null;
} // finishGame

GameObj.firstTurn = function () {
	var track, score, win, playerName, myPlayer;
	var i;
		
	// check if one or more players got dealt 31
	win = 0;
	for (i=0; i<this.playerCount; i++) {
		myPlayer = this.players[i];
		score = myPlayer.getScore ();
		if (score >= this.winningScore) {
			playerName = myPlayer.getPlayerName() + " (" +
										getCardsDesc( myPlayer.getAllCards() ) + ")";
			track == null ? track = playerName : track += ", " + playerName; 
			win++;
		}
	}
	if (win == 0) {
		// no winning hand dealt
		// regular game 
		
		// hide the screens so the player can take over for the opeing screen
		this.screens.hide();
		
		// pick player after the dealer as the first player to make a turn
		this.currentTurn = getNextPlayer (this.dealer); 
		
		// and go...
		this.players[this.currentTurn].yourTurn();
	}
	else if (win >= this.playerCount) {
		// all players have 31 - deal again
		this.finishGame("All winners");
	} 
	else if (win > 0) {
		// one or more winners
		win == 1 ? track += " wins." : track += " win.";
		// this.trackGame ("message", track);
		this.finishGame(track);
	}
} // firstTurn

GameObj.nextTurn = function () {
	var score, knockScore, myPlayer, track, win, i;
	
	score = this.players[this.currentTurn].getScore ();

	if (score < this.winningScore ) {
		this.currentTurn = getNextPlayer (this.currentTurn);
		
		// game over one round after knowcking?	
		if (this.knockPlayer != null && this.knockPlayer == this.currentTurn) {
			myPlayer 		= this.players[this.knockPlayer];
			knockScore 	= myPlayer.getScore ();
			win = true;
			
			for (i=0; i<this.playerCount; i++) {
				if (i != this.currentTurn ) {
					score = this.players[i].getScore ();
					if (score >= knockScore) {
						win = false;
						break;
					}
				}
			}
			track = myPlayer.getPlayerName();
			
			if (win) { 
				// knocker wins
				track += " (" +	getCardsDesc(myPlayer.getAllCards() ) 
									+ ") wins with " + knockScore + "!";
				this.finishGame(track);
			} else {
				// knocker loses
				track += " loses after knocking.";
				this.finishGame( track, this.knockPlayer);
			}
			
		} else {
			// no game ending situation
			
			if (this.passCount >= this.playerCount) {
				// all players have passed, discard stack, get 3 new cards
				var newCards = this.getNewCards ();		
				
				this.passCount = 0;
				this.trackGame ("newCards", newCards);
			}
			
			// find all moves since the players last move in the log 
			// and pass it to the player
			i = 0;
			
			var moves = [];
			var move = this.gameLog[i++];
			
			while (move != null && ! ( move.type == "move" 
								&& move.player == this.players[this.currentTurn] ) )
			{
				moves.unshift (move);
				move = this.gameLog[i++];
			};
			
			this.players[this.currentTurn].yourTurn(moves);
		}
	}
	else {
		// game over - a winner
		var myPlayer = this.players[this.currentTurn]
		track = myPlayer.getPlayerName();
		track += " (" + getCardsDesc (myPlayer.getAllCards() ) + ") ";
		track += " wins with 31!";
		this.finishGame(track);
	}
} // nextTurn

// done

delete GameObj;