import java.io.OutputStream;
import java.util.Hashtable;
import java.util.Properties;

/**
 * A PocketService is a lightweight component designed to provide a single,
 * simple service for an arbitrary client. It runs along with its siblings
 * inside a PocketServer which manages and executes it.
 *
 * @author Christian Cantrell
 * @version $Id: PocketService.java,v 1.9 2001/09/09 05:05:39 cantrell Exp $
 */
public abstract class PocketService
{
    /**
     * Debug flag which is configured from a properties file.
     */
    protected boolean debug;

    /**
     * Properties object containing initialization information.
     */
    protected Properties props;

    /**
     * Ask the service to do its work.
     *
     * @param request A Hashtable of parameters passed in from client.
     */
    protected abstract void service(PocketRequest request,
                                    PocketResponse response)
        throws PocketServiceException;

    /**
     * The static logger.
     */
    private PocketLogger logger;

    /**
     * Initialize the implementation of the PocketServer and pass it a
     * Properties object with conif info. Then calls initSupport() which
     * may be overridden from additional initialization.
     */
    protected void init(Properties props)
        throws PocketInitException
    {
        this.props = props;
        this.logger = PocketLogger.getInstance();
        initSupport();
    }

    /**
     * Override this if your service needs to do initialization specific to it.
     */
    protected void initSupport()
        throws PocketInitException
    {
        return;
    } 
}
