#include "PocketServiceParameter.as"


/*
 * Encapsulates a request to the PocketServer. Only supports http://.
 */

/*
 * Create a new PocketServiceRequest and pass it a URL and a port number.
 */
function PocketServiceRequest(server,port)
{
    this.params = new Array();
    this.server = server;
    this.port = port;
}

/*
 * Set the name of the pocket service to envoke.
 */
function setService(service)
{
    this.service = service;
}

/*
 * Clears the PocketServiceParameters so this object can be reused.
 */
function clearParameters()
{
    this.params = new Array();
}

/*
 * Add a parameter to the request.
 */
function addParameter(name, value)
{
    this.params[this.params.length] = new PocketServiceParameter(name,
                                                                 value);
}

/*
 * Make the HTTP request.
 *
 * @param listener Gets getResponse(doc) called when the data arrives from the
 *        server. If you'd rather overrice this object's getResponse(doc) method,
 *        just pass in null.
 */
function execute(listener)
{
    this.listener = listener;
    var req = "http://" +
              this.server + ":" +
              this.port + "/" +
              this.service +
              this.getQueryParameters();
    this.load(req);
}

/**
 * Simply calls setResponse on the listener.
 */
function pr_onData(doc)
{
    this.listener.setResponse(doc);
}

/*
 * Converts the HttpRequest into a String for the XMLSocket object's
 * send() method.
 */
function getQueryParameters()
{
    var qs = new String();
    for (var i = 0; i < this.params.length; ++i)
    {
        if (i == 0)
        {
            qs = qs.concat("?");
        }
        else
        {
            qs = qs.concat("&");
        }
        var p = this.params[i];
        qs = qs.concat(p.getName());
        qs = qs.concat("=");
        qs = qs.concat(p.getValue());
    }
    return qs;
}

PocketServiceRequest.prototype.__proto__          = XML.prototype;
PocketServiceRequest.prototype.addParameter       = addParameter;
PocketServiceRequest.prototype.setService         = setService;
PocketServiceRequest.prototype.getQueryParameters = getQueryParameters;
PocketServiceRequest.prototype.clearParameters    = clearParameters;
PocketServiceRequest.prototype.execute            = execute;
PocketServiceRequest.prototype.onData             = pr_onData;