import java.io.File;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.io.IOException;
import java.util.Properties;

/**
 * A PocketService which reads a file and writes its contents to the
 * OutputStream. The following parameters must be present in the request:
 * <ul>
 *   <li>
 *     fileName - The name of the file to read.
 *   </li>
 * </ul>
 *
 * @author Christian Cantrell
 * @version $Id: ReadFile.java,v 1.7 2001/12/21 05:45:17 cantrell Exp $
 */
public class ReadFile
    extends PocketService
{
    /**
     * The directory where files will be read. This must be specified by the
     * server rather than letting the client specify the entire path for
     * security reasons.
     */
    private String dir;

    protected void initSupport()
        throws PocketInitException
    {
        dir = props.getProperty("ReadFile.readBaseDir");
        if (dir == null)
        {
            throw new PocketInitException("The PocketService ReadFile cannot " +
                                          "be initialized w/o the property " +
                                          "ReadFile.readBaseDir defined."); 
        }

        if (!dir.endsWith(File.separator))
        {
            dir += File.separator;
        }
    }

    protected void service(PocketRequest request,
                           PocketResponse response)
        throws PocketServiceException
    {
        String fileName = request.getParameter("fileName");
        if (fileName == null)
        {
            throw new PocketServiceException("ReadFile does not have all the " +
                                             "information it needs to read " +
                                             "a file. You must pass in " +
                                             "fileName."); 
        }

        if (fileName.startsWith(File.separator))
        {
            fileName = fileName.substring(1,fileName.length());
        }

        fileName = dir + fileName;

        try
        {
            FileInputStream fileIn = new FileInputStream(fileName);
            InputStreamReader reader = new InputStreamReader(fileIn);
            char[] buffer = new char[1024];
            for (int cnt = reader.read(buffer);
                 cnt != -1;
                 cnt = reader.read(buffer))
            {
                response.write(buffer, 0, cnt);
            }
            response.flush();
            response.close();
            fileIn.close();
            reader.close();
            return; 
        }
        catch (IOException exception)
        {
            throw new PocketServiceException("Problem writing file [" +
                                             fileName + "] to OutputStream: " +
                                             exception);
        }
    }
}
